package mahjong.automata;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import mahjong.pta.Field;
import mahjong.pta.Obj;
import mahjong.pta.Type;

/**
 * @author Tian Tan
 * @author Yue Li
 */
public class DFAState {

	private final Set<Obj> objs;
	private final Set<Type> output;
	private final Map<Field, DFAState> nextMap;
	
	private boolean hasHashCode = false;
	private int hashCode;
	
	public DFAState(Set<Obj> objs, Set<Type> output) {
		this.objs = objs;
		this.output = output;
		this.nextMap = new HashMap<>();
	}
	
	public Set<Obj> getObjects() {
		return objs;
	}
	
	public Set<Type> getOutput() {
		return output;
	}
	
	void addTransition(Field f, DFAState nextState) {
		nextMap.put(f, nextState);
	}
	
	Map<Field, DFAState> getNextMap() {
		return nextMap;
	}
	
	/**
	 * Cache hash code.
	 */
	@Override
	public int hashCode() {
		if (!hasHashCode) {
			hashCode = objs.hashCode();
			hasHashCode = true;
		}
		return hashCode;
	}
	
	@Override
	public boolean equals(Object other) {
		if (this == other) {
			return true;
		}
		if (! (other instanceof DFAState)) {
			return false;
		}
		DFAState anoDFAState = (DFAState) other;
		return getObjects().equals(anoDFAState.getObjects());
	}
	
	@Override
	public String toString() {
		return getObjects().stream()
				.map(obj -> obj == null ? "null" :String.valueOf(obj.getID()))
				.sorted()
				.collect(Collectors.toSet())
				.toString();
	}
	
}
