package mahjong.automata;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import mahjong.fpg.FieldPointstoGraph;
import mahjong.pta.Field;
import mahjong.pta.Obj;
import mahjong.pta.Type;

/**
 * @author Tian Tan
 * @author Yue Li
 */
public class NFA {

	private static final Obj deadState = null;
	
	private Obj q0;
	private FieldPointstoGraph fpg;
	
	public NFA(Obj q0, FieldPointstoGraph fpg) {
		this.q0 = q0;
		this.fpg = fpg;
	}
	
	/**
	 * This method on-the-fly computes set of states.
	 * @return Set of states. Does not contains dead state.
	 */
	public Set<Obj> getStates() {
		Set<Obj> states = new HashSet<>();
		Stack<Obj> stack = new Stack<>();
		stack.push(q0);
		while (!stack.isEmpty()) {
			Obj s = stack.pop();
			if (!states.contains(s)) {
				states.add(s);
				outEdgesOf(s).forEach(field -> {
					nextStates(s, field).stream()
							.filter(obj -> !states.contains(obj))
							.forEach(stack::push);
				});
			}
		}
		return states;
	}
	
	public Obj getStartState() {
		return q0;
	}
	
	public Obj getDeadState() {
		return deadState;
	}
	
	public Set<Obj> nextStates(Obj obj, Field f) {
		if (isDeadState(obj) || !fpg.hasFieldPointer(obj, f)) {
			return Collections.singleton(deadState);
		} else {
			return fpg.pointsTo(obj, f);
		}
	}
	
	public boolean isDeadState(Obj obj) {
		return obj == deadState;
	}
	
	public Set<Field> outEdgesOf(Obj obj) {
		if (isDeadState(obj)) {
			return Collections.emptySet();
		} else {
			return fpg.outFieldsOf(obj);
		}
	}
	
	public Type outputOf(Obj obj) {
		if (isDeadState(obj)) {
			return null;
		} else {
			return obj.getType();
		}
	}
	
}
