package mahjong.pta.doop;

import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import mahjong.pta.Field;
import mahjong.pta.Obj;
import mahjong.pta.PTAProvider;
import mahjong.util.Triple;

/**
 * @author Tian Tan
 * @author Yue Li
 */
public class DoopPTAProvider implements PTAProvider {
	
	private static final String ARR_FIELD = "@ARRAY";
	
	private final DataBase db;
	private final ObjManager objManager;
	private final FieldManager fieldManager;
	
	public DoopPTAProvider(DataBase db) {
		this.db = db;
		this.objManager = new ObjManager(db);
		this.fieldManager = new FieldManager();
	}
	
	@Override
	public Iterator<Obj> objIterator() {
		return new ObjIterator();
	}
	
	@Override
	public Iterator<Triple<Obj, Field, Obj>> fptIterator() {
		return new FPTIterator();
	}

	private class ObjIterator implements Iterator<Obj> {

		private Iterator<List<String>> objIter;
		
		private ObjIterator() {
			objIter = db.query("OBJ").iterator();
		}
		
		@Override
		public boolean hasNext() {
			return objIter.hasNext();
		}

		@Override
		public Obj next() {
			if (hasNext()) {
				List<String> obj = objIter.next();
				return objManager.get(obj.get(0));
			} else {
				throw new NoSuchElementException();
			}
		}

	}

	private class FPTIterator implements Iterator<Triple<Obj, Field, Obj>> {
		
		private Iterator<List<String>> fptIter;
		private Iterator<List<String>> aptIter;
		
		private FPTIterator() {
			fptIter = db.query("IFPT").iterator();
			aptIter = db.query("APT").iterator();
		}

		@Override
		public boolean hasNext() {
			return fptIter.hasNext() || aptIter.hasNext();
		}

		@Override
		public Triple<Obj, Field, Obj> next() {
			if (fptIter.hasNext()) {
				List<String> fpt = fptIter.next();
				Obj baseObj = objManager.get(fpt.get(0));
				Field field = fieldManager.get(fpt.get(1));
				Obj obj = objManager.get(fpt.get(2));
				return new Triple<Obj, Field, Obj>(baseObj, field, obj);
			} else if (aptIter.hasNext()) {
				List<String> apt = aptIter.next();
				Obj array = objManager.get(apt.get(0));
				Field field = fieldManager.get(ARR_FIELD);
				Obj obj = objManager.get(apt.get(1));
				return new Triple<Obj, Field, Obj>(array, field, obj);
			} else {
				throw new NoSuchElementException();
			}
		}
	}
}
